@@ -74,7 +74,7 @@ uint32_t AddNameToNameTable(std::string &ioNameTable, const std::string &inName)
 	{
 		if (inName == p)
 		{
-			result = p - ioNameTable.c_str();
+			result = static_cast<uint32_t>(p - ioNameTable.c_str());
 			break;
 		}
 
@@ -83,7 +83,7 @@ uint32_t AddNameToNameTable(std::string &ioNameTable, const std::string &inName)
 
 	if (p >= e)
 	{
-		result = ioNameTable.length();
+		result = static_cast<uint32_t>(ioNameTable.length());
 		ioNameTable.append(inName);
 		ioNameTable.append("\0", 1);
 	}
@@ -226,7 +226,7 @@ uint32_t WriteDataAligned(std::ofstream &inStream, const void *inData, uint32_t
 			inStream.put('\0');
 	}
 
-	return inStream.tellp();
+	return static_cast<uint32_t>(inStream.tellp());
 }
 
 // --------------------------------------------------------------------
@@ -516,6 +516,8 @@ void MELFObjectFileImp<ELF_CLASSXX, ELF_DATAXX>::Write(std::ofstream &f)
 // --------------------------------------------------------------------
 // PE/COFF
 
+#ifndef IMAGE_FILE_MACHINE_AMD64
+
 enum COFF_Machine : uint16_t
 {
 	IMAGE_FILE_MACHINE_AMD64 = 0x8664,
@@ -529,9 +531,34 @@ enum COFF_HeaderCharacteristics : uint16_t
 	IMAGE_FILE_LINE_NUMS_STRIPPED = 0x0004
 };
 
+enum COFF_SectionHeaderCharacteristics : uint32_t
+{
+	IMAGE_SCN_CNT_CODE = 0x00000020,
+	IMAGE_SCN_LNK_INFO = 0x00000200,
+	IMAGE_SCN_LNK_REMOVE = 0x00000800,
+	IMAGE_SCN_CNT_INITIALIZED_DATA = 0x00000040,
+	IMAGE_SCN_CNT_UNINITIALIZED_DATA = 0x00000080,
+	IMAGE_SCN_ALIGN_1BYTES = 0x00100000,
+	IMAGE_SCN_ALIGN_8BYTES = 0x00400000,
+	IMAGE_SCN_ALIGN_16BYTES = 0x00500000,
+	IMAGE_SCN_MEM_EXECUTE = 0x20000000,
+	IMAGE_SCN_MEM_READ = 0x40000000,
+	IMAGE_SCN_MEM_WRITE = 0x80000000
+};
+
+
+enum COFF_StorageClass : uint8_t
+{
+	IMAGE_SYM_CLASS_EXTERNAL = 0x02,
+	IMAGE_SYM_CLASS_STATIC = 0x03,
+	IMAGE_SYM_CLASS_FILE = 0x67
+};
+
+#endif
+
 struct COFF_Header
 {
-	COFF_Machine machine = IMAGE_FILE_MACHINE_AMD64;
+	uint16_t machine = IMAGE_FILE_MACHINE_AMD64;
 	uint16_t numberOfSections;
 	uint32_t timeDateStamp = 0;
 	uint32_t pointerToSymbolTable;
@@ -552,21 +579,6 @@ union COFF_Name
 
 static_assert(sizeof(COFF_Header) == 20, "COFF_Header size should be 20 bytes");
 
-enum COFF_SectionHeaderCharacteristics : uint32_t
-{
-	IMAGE_SCN_CNT_CODE = 0x00000020,
-	IMAGE_SCN_LNK_INFO = 0x00000200,
-	IMAGE_SCN_LNK_REMOVE = 0x00000800,
-	IMAGE_SCN_CNT_INITIALIZED_DATA = 0x00000040,
-	IMAGE_SCN_CNT_UNINITIALIZED_DATA = 0x00000080,
-	IMAGE_SCN_ALIGN_1BYTES = 0x00100000,
-	IMAGE_SCN_ALIGN_8BYTES = 0x00400000,
-	IMAGE_SCN_ALIGN_16BYTES = 0x00500000,
-	IMAGE_SCN_MEM_EXECUTE = 0x20000000,
-	IMAGE_SCN_MEM_READ = 0x40000000,
-	IMAGE_SCN_MEM_WRITE = 0x80000000
-};
-
 struct COFF_SectionHeader
 {
 	COFF_Name name;
@@ -583,13 +595,6 @@ struct COFF_SectionHeader
 
 static_assert(sizeof(COFF_SectionHeader) == 40, "Section headers should be 40 bytes");
 
-enum COFF_StorageClass : uint8_t
-{
-	IMAGE_SYM_CLASS_EXTERNAL = 0x02,
-	IMAGE_SYM_CLASS_STATIC = 0x03,
-	IMAGE_SYM_CLASS_FILE = 0x67
-};
-
 /// \brief COFF symbol table entry, should be 18 bytes... right...
 struct COFF_Symbol
 {
@@ -619,13 +624,13 @@ struct MCOFFObjectFileImp : public MObjectFileImp
 
 	virtual void Write(std::ofstream &inFile) override;
 
-	MCOFFObjectFileImp(COFF_Machine machine)
+	MCOFFObjectFileImp(uint16_t machine)
 		: MObjectFileImp()
 		, mMachine(machine)
 	{
 	}
 
-	COFF_Machine mMachine;
+	uint16_t mMachine;
 };
 
 void MCOFFObjectFileImp::Write(std::ofstream &f)
@@ -696,13 +701,13 @@ void MCOFFObjectFileImp::Write(std::ofstream &f)
 	{
 		symbols.emplace_back(
 			COFF_Symbol{
-				addName(name),
+				addName(mMachine == IMAGE_FILE_MACHINE_I386 ? '_' + name : name),
 				offset - rawDataOffset,
 				1,
 				0,
 				IMAGE_SYM_CLASS_EXTERNAL });
 
-		offset = WriteDataAligned(f, data.data(), data.size());
+		offset = WriteDataAligned(f, data.data(), static_cast<uint32_t>(data.size()));
 	}
 
 	auto dataSize = offset - rawDataOffset;
@@ -715,7 +720,7 @@ void MCOFFObjectFileImp::Write(std::ofstream &f)
 	using namespace std::literals;
 	std::string package_string = kProjectName + " "s + kVersionNumber;
 
-	offset = WriteDataAligned(f, package_string.data(), package_string.length());
+	offset = WriteDataAligned(f, package_string.data(), static_cast<uint32_t>(package_string.length()));
 
 	auto rawData2Size = offset - rawData2Offset;
 	auto symbolTableOffset = offset;
@@ -755,9 +760,9 @@ void MCOFFObjectFileImp::Write(std::ofstream &f)
 	for (auto &sym : symbols)
 		WriteDataAligned(f, &sym, 18);
 
-	uint32_t strTabSize = strtab.size() + 4;
+	uint32_t strTabSize = static_cast<uint32_t>(strtab.size() + 4);
 	WriteDataAligned(f, &strTabSize, sizeof(strTabSize));
-	WriteDataAligned(f, strtab.data(), strtab.size() + 1);
+	WriteDataAligned(f, strtab.data(), static_cast<uint32_t>(strtab.size() + 1));
 
 	// write section headers
 	f.seekp(sectionHeaderStart);
@@ -767,7 +772,7 @@ void MCOFFObjectFileImp::Write(std::ofstream &f)
 	f.seekp(0);
 	header.numberOfSections = sizeof(sectionHeaders) / sizeof(COFF_SectionHeader);
 	header.pointerToSymbolTable = symbolTableOffset;
-	header.numberOfSymbols = symbols.size();
+	header.numberOfSymbols = static_cast<uint32_t>(symbols.size());
 	WriteDataAligned(f, &header, sizeof(header));
 }
 
@@ -808,7 +813,7 @@ class MObjectFile
 	}
 #endif
 
-	MObjectFile(COFF_Machine machine)
+	MObjectFile(uint16_t machine)
 		: mImpl(new MCOFFObjectFileImp(machine))
 	{
 	}
@@ -880,13 +885,13 @@ void MResourceFile::AddEntry(fs::path inPath, const char *inData, uint32_t inSiz
 		{
 			mrsrc::rsrc_imp child = {};
 
-			child.m_name = mName.size();
+			child.m_name = static_cast<uint32_t>(mName.size());
 
 			std::string n = p->string();
 			copy(n.begin(), n.end(), std::back_inserter(mName));
 			mName.push_back(0);
 
-			mIndex[node].m_child = mIndex.size();
+			mIndex[node].m_child = static_cast<uint32_t>(mIndex.size());
 			mIndex.push_back(child);
 
 			node = mIndex[node].m_child;
@@ -916,13 +921,13 @@ void MResourceFile::AddEntry(fs::path inPath, const char *inData, uint32_t inSiz
 			// not found, create it
 			mrsrc::rsrc_imp n = {};
 
-			n.m_name = mName.size();
+			n.m_name = static_cast<uint32_t>(mName.size());
 
 			std::string s = p->string();
 			copy(s.begin(), s.end(), back_inserter(mName));
 			mName.push_back(0);
 
-			node = mIndex.size();
+			node = static_cast<uint32_t>(mIndex.size());
 			mIndex[next].m_next = node;
 			mIndex.push_back(n);
 
@@ -934,7 +939,7 @@ void MResourceFile::AddEntry(fs::path inPath, const char *inData, uint32_t inSiz
 	assert(node < mIndex.size());
 
 	mIndex[node].m_size = inSize;
-	mIndex[node].m_data = mData.size();
+	mIndex[node].m_data = static_cast<uint32_t>(mData.size());
 
 	copy(inData, inData + inSize, back_inserter(mData));
 	while ((mData.size() % 8) != 0)
@@ -957,7 +962,7 @@ void MResourceFile::Add(const fs::path &inPath, const fs::path &inFile)
 	}
 	else
 	{
-		if (VERBOSE)
+		if (VERBOSE > 0)
 			std::cerr << "adding " << inFile << " as " << inPath / inFile.filename() << std::endl;
 
 		std::ifstream f(inFile, std::ios::binary);
@@ -967,7 +972,7 @@ void MResourceFile::Add(const fs::path &inPath, const fs::path &inFile)
 
 		std::filebuf *b = f.rdbuf();
 
-		uint32_t size = b->pubseekoff(0, std::ios::end, std::ios::in);
+		uint32_t size = static_cast<uint32_t>(b->pubseekoff(0, std::ios::end, std::ios::in));
 		b->pubseekoff(0, std::ios::beg, std::ios::in);
 
 		std::vector<char> text(size);
@@ -981,9 +986,9 @@ void MResourceFile::Add(const fs::path &inPath, const fs::path &inFile)
 
 void MResourceFile::Write(MObjectFile &obj)
 {
-	obj.AddGlobal(mPrefix + "Index", mIndex.data(), mIndex.size() * sizeof(mrsrc::rsrc_imp));
-	obj.AddGlobal(mPrefix + "Data", mData.data(), mData.size());
-	obj.AddGlobal(mPrefix + "Name", mName.data(), mName.size());
+	obj.AddGlobal(mPrefix + "Index", mIndex.data(), static_cast<uint32_t>(mIndex.size() * sizeof(mrsrc::rsrc_imp)));
+	obj.AddGlobal(mPrefix + "Data", mData.data(), static_cast<uint32_t>(mData.size()));
+	obj.AddGlobal(mPrefix + "Name", mName.data(), static_cast<uint32_t>(mName.size()));
 }
 
 // --------------------------------------------------------------------
@@ -1001,6 +1006,7 @@ int main(int argc, char *argv[])
 		mcfp::make_option("header", "This will print out the header file you need to include in your program to access your resources"),
 		mcfp::make_option<std::string>("root", "Root path for the stored data (in the final resource data structure"),
 		mcfp::make_option<std::string>("resource-prefix", "gResource", "Prefix for the name of the global variables, default is gResource"),
+		mcfp::make_option<std::string>("depends,d", "Print a list of files the resource depends upon to the file specified"),
 
 #if __has_include(<elf.h>)
 		mcfp::make_option<int>("elf-machine", "The ELF machine type to use, default is same as this machine. Use one of the values from elf.h"),
@@ -1055,12 +1061,56 @@ int main(int argc, char *argv[])
 
 	VERBOSE = config.count("verbose");
 
+	if (config.has("depends"))
+	{
+		std::vector<std::string> files;
+		
+		for (auto p : config.operands())
+		{
+			if (fs::is_directory(p))
+			{
+				for (auto i = fs::recursive_directory_iterator(p); i != fs::recursive_directory_iterator(); ++i)
+					files.emplace_back(i->path().string());
+			}
+			else
+				files.emplace_back(p);
+		}
+
+		std::ofstream depends(config.get("depends"));
+		if (not depends.is_open())
+			throw std::runtime_error("Could not open depends file for output");
+
+		depends << config.get("output") << ": ";
+		for (size_t i = 0; i < files.size(); ++i)
+		{
+			auto &file = files[i];
+			
+			for (char c : { ' ', '$', '#' })
+			{
+				for (auto s = file.find(c); s != std::string::npos; s = file.find(c, s))
+				{
+					file.insert(file.begin() + s, '\\');
+					s += 2;
+				}
+			}
+
+			depends << files[i];
+			if (i + 1 < files.size())
+				depends << " \\\n\t";
+		}
+
+		depends << "\n";
+		depends.close();
+
+		exit(0);
+	}
+
 	// --------------------------------------------------------------------
 	// find out the native format. Simply look at how we were assembled ourselves
 
 	int elf_machine = EM_NONE, elf_class = 0, elf_data = 0, elf_flags = 0, elf_abi = 0;
 
-#if not defined(_MSC_VER)
+#if not defined(_WIN32)
 	char exePath[PATH_MAX + 1];
 #if __linux or __linux__
 	elf_abi = ELFOSABI_LINUX;
@@ -1069,6 +1119,8 @@ int main(int argc, char *argv[])
 	elf_abi = ELFOSABI_FREEBSD;
 	int r = strlen(argv[0]);
 	strcpy(exePath, argv[0]);
+#else
+#error "Unsupported OS, sorry..."
 #endif
 
 	if (r > 0)
@@ -1137,12 +1189,15 @@ int main(int argc, char *argv[])
 		for (fs::path i : config.operands())
 			rsrcFile.Add(ns, i);
 
+		if (not config.has("output"))
+			return 0;
+
 		std::ofstream file(config.get<std::string>("output"), std::ios::binary);
 		if (not file.is_open())
 			throw std::runtime_error("Could not open output file for writing");
 
-		COFF_Machine win_machine = {};
-#if defined(_MSC_VER)
+		uint16_t win_machine = {};
+#if defined(_WIN32)
 #if defined(_M_AMD64)
 		win_machine = IMAGE_FILE_MACHINE_AMD64;
 #elif defined(_M_ARM64)
